/*-----------------*/
/* blk2.c          */
/* RAM disk driver */
/*-----------------*/

#include "../h/types.h"
#include "../h/param.h"
#include "../h/buf.h"
#include "../h/io.h"
#include "../h/ioctl.h"
#include "../h/errno.h"
#include "../h/conf.h"
#include "../h/sysmacros.h"
#include "../h/diskio.h"
#include "../h/dir.h"
#include "../h/moddefs.h"
#include "../h/mod_conf.h"
#include "../wsio/wsio.h"/* if KERN def'd expects eeprom.h in wsio */

#define RAMDISK_SIZE2*1024*1024

static caddr_t ramptr = NULL;
static uint32_t ram_size = 0;
static int nblks = 0;
static space_t ramsid;

int blk2_install();
void blk2_init();
int blk2_open();
void blk2_close();
int blk2_ioctl();
int blk2_read();
int blk2_write();


void blk2_strategy();
extern int (*dev_init)();
int (*blk2_saved_dev_init)();
int blk2_dlkm = 0;

extern struct mod_operations gio_mod_ops;
static int blk2_load();
static int blk2_unload();

/* static */ struct mod_type_data blk2_drv_link = {
"blk2", NULL
};

/* static */ struct modlink blk2_mod_link[] = {
{ &gio_mod_ops, &blk2_drv_link },
{ NULL, NULL }
};

/* static */ struct mod_conf_data mc = { MCD_VERSION,0 };

/* static */ struct modwrapper blk2_wrapper = {
MODREV, blk2_load, blk2_unload, NULL,
&mc, blk2_mod_link
};

static drv_info_t blk2_drv_info = {
"blk2disc",
"pseudo",
DRV_CHAR|DRV_BLOCK| DRV_PSEUDO,
-1,
-1,
NULL,
NULL,
NULL,
};static drv_ops_t blk2_drv_ops = {
blk2_open,
blk2_close,
blk2_strategy,
NULL,
NULL,
NULL,
blk2_read,
blk2_write,
blk2_ioctl,
NULL,
NULL,
NULL,
NULL,
NULL,
NULL,
0,
};

static wsio_drv_data_t blk2_data = {
"blk2_ramdisk",
T_UNKNOWN,
DRV_CONVERGED,
NULL,
NULL
};

static wsio_drv_info_t blk2_wsio_drv_info = {
&blk2_drv_info,
&blk2_drv_ops,
&blk2_data
};


/*---------------*/
/*  BLK2_INSTALL */
/*---------------*/

int blk2_install() {
blk2_saved_dev_init = dev_init;
dev_init = blk2_init;
blk2_dlkm = 0;
return (wsio_install_driver(&blk2_wsio_drv_info));
}

/*------------*/
/* BLK2_LOAD  */
/*------------*/
int blk2_load(drv_info_arg)


void *drv_info_arg;
{if (drv_info_arg == NULL) {
printf("blk2_load null ptr passed in\n");
return 1;
}blk2_wsio_drv_info.drv_info = (struct drv_info *)drv_info_arg;
if (!wsio_install_driver(&blk2_wsio_drv_info)) {
printf("blk2_load wsio install driver failed\n");
return ENXIO;;
}blk2_dlkm = 1;
blk2_init();
return 0;
}

/*-------------*/
/* BLK2_UNLOAD */
/*-------------*/
int blk2_unload(drv_info_arg)
void *drv_info_arg;
{blk2_wsio_drv_info.drv_info = (struct drv_info *)drv_info_arg;
wsio_uninstall_driver(&blk2_wsio_drv_info);
if (ramptr) {
sys_memfree(ramptr,(long)ram_size);
}return 0;
}

/*------------*/
/* BLK2_INIT  */
/*------------*/
void
blk2_init()
{ram_size = RAMDISK_SIZE;
ramptr = sys_memall((long)ram_size);
if (ramptr) {
ramsid = ldsid(ramptr);
nblks = RAMDISK_SIZE >> DEV_BSHIFT;
} else {



nblks = 0;
ram_size = 0;
}if (!blk2_dlkm)
(*blk2_saved_dev_init)();
}

/*------------*/
/* BLK2_OPEN  */
/*------------*/
blk2_open(dev,flag)
dev_t dev;
int flag;
{if (ram_size == 0)
return ENOSPC;
return 0;
}/*-----------*/
/* BLK2_READ */
/*-----------*/
int
blk2_read(dev,uio)
dev_t dev;
struct uio *uio;
{if ((uio->uio_offset & DEV_BMASK) &&
((uio->uio_fpflags & FBLKSEEK) == 0))
return(ENXIO);
return physio(blk2_strategy,NULL,dev,B_READ,minphys,uio);
}int blk2_write(dev,uio)
dev_t dev;
struct uio *uio;
{if ((uio->uio_offset & DEV_BMASK) &&
        ((uio->uio_fpflags & FBLKSEEK) == 0))
        return(ENXIO);
    return physio(blk2_strategy, NULL, dev, B_WRITE, minphys, uio);
}

/*---------------*/

/* BLK2_STRATEGY */
/*---------------*/

void
blk2_strategy(bp)
struct buf *bp;
{size_t size;
caddr_t start;
space_t bp_sid;

bp_sid = bvtospace(bp,bp->b_un.b_addr);
start = ramptr + bp->b_offset;
/* check transfer in volume and nonzero length */
if ((start >= ramptr + ram_size) || (ram_size == 0)) {
 bp->b_resid = bp->b_bcount;
                bp->b_flags |= B_ERROR;
                bp->b_error = ENOSPC;
iodone(bp);
                return;
        }
/* truncate if transfer goes past end of volume */
size = MIN((int)((uint32_t)ramptr + ram_size) - (uint32_t)start,
bp->b_bcount);
/* abort if not a multiple of block size */
if (bp->b_bcount & DEV_BMASK) {
                bp->b_resid = bp->b_bcount;
                bp->b_flags |= B_ERROR;
                bp->b_error = ENXIO;
                iodone(bp);
                return;
        }
        /* do the transfer */
        if (bp->b_flags & B_READ)
             privlbcopy(ramsid,start,bp_sid,bp->b_un.b_addr,bp->b_bcount);
        else privlbcopy(bp_sid,bp->b_un.b_addr,ramsid,start,bp->b_bcount);
        iodone(bp);
}

/*------------*/
/* BLK2_IOCTL */

/*------------*/
int
blk2_ioctl(dev,cmd,arg,flag)
dev_t dev;
int cmd;
caddr_t arg; /* was int arg, changed to caddr_t for 64-bit */
int flag;
{capacity_type *cap;
disk_describe_type *describe;
int size =  (cmd & IOCSIZE_MASK) >> 16;

switch (cmd & IOCCMD_MASK) {
case DIOC_DESCRIBE & IOCCMD_MASK:
if (cmd != DIOC_DESCRIBE && size != 32)
          return EINVAL;
describe = (disk_describe_type *)arg;
(void) strcpy(describe->model_num,"BLK2_DISK");
describe->intf_type = UNKNOWN_INTF;
describe->maxsva = nblks - 1;
describe->lgblksz = DEV_BSIZE;
describe->dev_type = DISK_DEV_TYPE;
describe->flags = 0;
printf("blk2:DIOC_DESCRIBE maxsva = 0x%x,lgblksz = 0x%x\n",
describe->maxsva,describe->lgblksz);
return 0;
}

switch (cmd) {
case DIOC_CAPACITY:
cap = (capacity_type *) arg;
cap->lba = nblks;
printf("blk2_ioctl:DIOC_CAPACITY cap->lba = 0x%x\n",cap->lba);
return 0;
}return ENXIO;
}/*-------------*/
/* BLK2_CLOSE */
/*-------------*/

void
blk2_close(dev)
dev_t dev;
{return;
}
